"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createHeap = void 0;
var sanitize_1 = require("./sanitize");
var createHeap = function (handleInvocation, logger, defaultSource) {
    if (defaultSource === void 0) { defaultSource = null; }
    var logSource = defaultSource === null || defaultSource === void 0 ? void 0 : defaultSource.name;
    var sanitizedDefaultSource = (0, sanitize_1.sanitizeSourceInfo)(defaultSource, logger, logSource);
    return {
        startRecording: function (environmentId, options) {
            if (arguments.length === 0) {
                logger.warn('startRecording failed because environmentId was omitted.', logSource);
                return;
            }
            if (!(0, sanitize_1.isNonEmptyString)(environmentId)) {
                logger.warn('startRecording failed because environmentId was not a string.', logSource);
                return;
            }
            handleInvocation('startRecording', { environmentId: environmentId, options: options || {} });
        },
        stopRecording: function () {
            handleInvocation('stopRecording', {});
        },
        track: function (event, properties, timestamp, sourceInfo) {
            if (arguments.length === 0) {
                logger.warn('track failed because event was omitted.', logSource);
                return;
            }
            if (!(0, sanitize_1.isNonEmptyString)(event)) {
                logger.warn('track failed because event was not a string.', logSource);
                return;
            }
            var sanitizedProperties = (0, sanitize_1.sanitizeProperties)(properties, logger, logSource);
            var sanitizedSourceInfo = (0, sanitize_1.sanitizeSourceInfo)(sourceInfo, logger, logSource) || sanitizedDefaultSource;
            var javascriptEpochTimestamp = (0, sanitize_1.sanitizeTimestamp)(timestamp, logger, logSource);
            // Build the event.
            var args = { event: event };
            if (sanitizedProperties) {
                args.properties = sanitizedProperties;
            }
            if (javascriptEpochTimestamp) {
                args.javascriptEpochTimestamp = javascriptEpochTimestamp;
            }
            if (sanitizedSourceInfo) {
                args.sourceLibrary = sanitizedSourceInfo;
            }
            handleInvocation('track', args);
        },
        identify: function (identity) {
            if (arguments.length === 0) {
                logger.warn('identify failed because identity was omitted.', logSource);
                return;
            }
            if (!(0, sanitize_1.isNonEmptyString)(identity)) {
                logger.warn('identify failed because identity was not a string.', logSource);
                return;
            }
            handleInvocation('identify', { identity: identity });
        },
        resetIdentity: function () {
            handleInvocation('resetIdentity', {});
        },
        addUserProperties: function (properties) {
            var sanitizedProperties = (0, sanitize_1.sanitizeProperties)(properties, logger, logSource);
            if (!sanitizedProperties) {
                return; // Already logged.
            }
            if (Object.keys(sanitizedProperties).length === 0) {
                logger.warn('addUserProperties was called without any valid properties.');
                return;
            }
            handleInvocation('addUserProperties', { properties: sanitizedProperties });
        },
        addEventProperties: function (properties) {
            var sanitizedProperties = (0, sanitize_1.sanitizeProperties)(properties, logger, logSource);
            if (!sanitizedProperties) {
                return; // Already logged.
            }
            if (Object.keys(sanitizedProperties).length === 0) {
                logger.warn('addUserProperties was called without any valid properties.');
                return;
            }
            handleInvocation('addEventProperties', { properties: sanitizedProperties });
        },
        removeEventProperty: function (name) {
            if (arguments.length === 0) {
                logger.warn('removeEventProperty failed because name was omitted.', logSource);
                return;
            }
            if (!(0, sanitize_1.isNonEmptyString)(name)) {
                logger.warn('removeEventProperty failed because name was not a string.', logSource);
                return;
            }
            handleInvocation('removeEventProperty', { name: name });
        },
        clearEventProperties: function () {
            handleInvocation('clearEventProperties', {});
        },
        setLogLevel: function (logLevel) {
            logger.setLogLevel(logLevel);
        },
        getSessionId: function () {
            return handleInvocation('sessionId', {});
        },
        fetchSessionId: function () {
            return handleInvocation('fetchSessionId', {});
        },
        getUserId: function () {
            return handleInvocation('userId', {});
        },
        getIdentity: function () {
            return handleInvocation('identity', {});
        },
    };
};
exports.createHeap = createHeap;
